AUDIO_DIR = './asset/audio'
VIDEO_DIR = './asset/video'
INTERVAL_DIR = './asset/time'
UNCOMPRESSED_VIDEO_DIR = './asset/uncompressed-video'
PROCESSED_VIDEO_DIR = './asset/processed-video'
AUDIO_SPLIT_DIR = './asset/separated-audio'
VIDEO_SPLIT_DIR = './asset/separated-video'
WORDLIST_DIR = './asset/wordlists'
MERGE_SPLIT_DIR = './asset/merged-video'

LANDMARK_DETECTOR_PATH = './src/FaceDetection/shape_predictor_68_face_landmarks.dat'
LIP_MARGIN = 0.3
LIP_CROP_SIZE = (112, 112)

WORDS_PER_BATCH = 25
DEFAULT_SAMPLING_RATE = 44100  # Hz
SPEAKER_ID_LENGTH = 3
BATCH_ID_LENGTH = 2
